"""EmailUser tests."""
from mock import patch
import os
import re
from unittest import skipIf, skipUnless

import django
from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.middleware import AuthenticationMiddleware
from django.core import mail
from django.core import management
from django.core.urlresolvers import reverse
from django.db import connection
from django.forms.fields import Field
from django.http import HttpRequest
from django.test import TestCase
from django.test.utils import override_settings
from django.utils import timezone
from django.utils.encoding import force_text
from django.utils.translation import ugettext as _

from .forms import EmailUserChangeForm, EmailUserCreationForm

try:
    from django.contrib.auth.middleware import SessionAuthenticationMiddleware
except ImportError:
    # Only available from Django 1.7, ignore the tests otherwise
    SessionAuthenticationMiddleware = None


class UserTest(TestCase):

    user_email = 'newuser@localhost.local'
    user_password = '1234'

    def create_user(self):
        """Create and return a new user with self.user_email as login and self.user_password as password."""
        return get_user_model().objects.create_user(self.user_email, self.user_password)

    def test_user_creation(self):
        # Create a new user saving the time frame
        right_now = timezone.now().replace(microsecond=0)  # MySQL doesn't store microseconds
        with patch.object(timezone, 'now', return_value=right_now):
            self.create_user()

        # Check user exists and email is correct
        self.assertEqual(get_user_model().objects.all().count(), 1)
        self.assertEqual(get_user_model().objects.all()[0].email, self.user_email)

        # Check date_joined and last_login dates
        self.assertEqual(get_user_model().objects.all()[0].date_joined, right_now)
        self.assertEqual(get_user_model().objects.all()[0].last_login, right_now)

        # Check flags
        self.assertTrue(get_user_model().objects.all()[0].is_active)
        self.assertFalse(get_user_model().objects.all()[0].is_staff)
        self.assertFalse(get_user_model().objects.all()[0].is_superuser)

    def test_user_get_full_name(self):
        user = self.create_user()
        self.assertEqual(user.get_full_name(), self.user_email)

    def test_user_get_short_name(self):
        user = self.create_user()
        self.assertEqual(user.get_short_name(), self.user_email)

    def test_email_user(self):
        # Email definition
        subject = "Email Subject"
        message = "Email Message"
        from_email = 'from@normal.com'

        user = self.create_user()

        # Test that no message exists
        self.assertEqual(len(mail.outbox), 0)

        # Send test email
        user.email_user(subject, message, from_email)

        # Test that one message has been sent
        self.assertEqual(len(mail.outbox), 1)

        # Verify that the email is correct
        self.assertEqual(mail.outbox[0].subject, subject)
        self.assertEqual(mail.outbox[0].body, message)
        self.assertEqual(mail.outbox[0].from_email, from_email)
        self.assertEqual(mail.outbox[0].to, [user.email])

    def test_email_user_kwargs(self):
        # valid send_mail parameters
        kwargs = {
            "fail_silently": False,
            "auth_user": None,
            "auth_password": None,
            "connection": None,
        }
        user = get_user_model()(email='foo@bar.com')
        user.email_user(
            subject="Subject here",
            message="This is a message", from_email="from@domain.com", **kwargs)
        # Test that one message has been sent.
        self.assertEqual(len(mail.outbox), 1)
        # Verify that test email contains the correct attributes:
        message = mail.outbox[0]
        self.assertEqual(message.subject, "Subject here")
        self.assertEqual(message.body, "This is a message")
        self.assertEqual(message.from_email, "from@domain.com")
        self.assertEqual(message.to, [user.email])


class UserManagerTest(TestCase):

    def test_create_user(self):
        email_lowercase = 'normal@normal.com'
        user = get_user_model().objects.create_user(email_lowercase)
        self.assertEqual(user.email, email_lowercase)
        self.assertFalse(user.has_usable_password())
        self.assertTrue(user.is_active)
        self.assertFalse(user.is_staff)
        self.assertFalse(user.is_superuser)

    def test_create_superuser(self):
        email_lowercase = 'normal@normal.com'
        password = 'password1234$%&/'
        user = get_user_model().objects.create_superuser(email_lowercase, password)
        self.assertEqual(user.email, email_lowercase)
        self.assertTrue(user.check_password, password)
        self.assertTrue(user.is_active)
        self.assertTrue(user.is_staff)
        self.assertTrue(user.is_superuser)

    def test_user_creation_is_active(self):
        # Create deactivated user
        email_lowercase = 'normal@normal.com'
        password = 'password1234$%&/'
        user = get_user_model().objects.create_user(email_lowercase, password, is_active=False)
        self.assertFalse(user.is_active)

    def test_user_creation_is_staff(self):
        # Create staff user
        email_lowercase = 'normal@normal.com'
        password = 'password1234$%&/'
        user = get_user_model().objects.create_user(email_lowercase, password, is_staff=True)
        self.assertTrue(user.is_staff)

    def test_create_user_email_domain_normalize_rfc3696(self):
        # According to http://tools.ietf.org/html/rfc3696#section-3
        # the "@" symbol can be part of the local part of an email address
        returned = get_user_model().objects.normalize_email(r'Abc\@DEF@EXAMPLE.com')
        self.assertEqual(returned, r'Abc\@DEF@example.com')

    def test_create_user_email_domain_normalize(self):
        returned = get_user_model().objects.normalize_email('normal@DOMAIN.COM')
        self.assertEqual(returned, 'normal@domain.com')

    def test_create_user_email_domain_normalize_with_whitespace(self):
        returned = get_user_model().objects.normalize_email('email\ with_whitespace@D.COM')
        self.assertEqual(returned, 'email\ with_whitespace@d.com')

    def test_empty_username(self):
        self.assertRaisesMessage(
            ValueError,
            'The given email must be set',
            get_user_model().objects.create_user, email=''
        )


@skipIf(django.VERSION < (1, 7, 0), 'Migrations not available in this Django version')
class MigrationsTest(TestCase):

    def test_makemigrations_no_changes(self):
        try:
            from StringIO import StringIO
        except ImportError:
            from io import StringIO

        with patch('sys.stdout', new_callable=StringIO) as mock:
            management.call_command('makemigrations', 'custom_user', dry_run=True)
        self.assertEqual(mock.getvalue(), 'No changes detected in app \'custom_user\'\n')

    @skipUnless(django.VERSION[:2] == (1, 7), 'Monkey patch only applied to Django 1.7')
    def test_monkey_patch_model_field(self):
        field_last_login = get_user_model()._meta.get_field('last_login')
        self.assertTrue(field_last_login.null)
        self.assertTrue(field_last_login.blank)

    @skipUnless(django.VERSION[:2] == (1, 7), 'Monkey patch only applied to Django 1.7')
    def test_monkey_patch_db_column(self):
        table_name = get_user_model()._meta.db_table
        cursor = connection.cursor()
        table_fields = connection.introspection.get_table_description(cursor, table_name)
        field = next(field for field in table_fields if field.name == 'last_login')  # pragma: no cover
        self.assertTrue(field.null_ok)

    @skipUnless(django.VERSION[:2] == (1, 7), 'Monkey patch only applied to Django 1.7')
    def test_monkey_patch_side_effects(self):
        # Check the parent model isn't affected from the monkey patch
        from django.contrib.auth.models import AbstractBaseUser
        field_last_login = AbstractBaseUser._meta.get_field('last_login')
        self.assertFalse(field_last_login.null)
        self.assertFalse(field_last_login.blank)


@skipIf(SessionAuthenticationMiddleware is None, "SessionAuthenticationMiddleware not available in this version")
class TestSessionAuthenticationMiddleware(TestCase):

    def setUp(self):
        self.user_email = 'test@example.com'
        self.user_password = 'test_password'
        self.user = get_user_model().objects.create_user(
            self.user_email,
            self.user_password)

        self.middleware_auth = AuthenticationMiddleware()
        self.middleware_session_auth = SessionAuthenticationMiddleware()
        self.assertTrue(self.client.login(
            username=self.user_email,
            password=self.user_password,
        ))
        self.request = HttpRequest()
        self.request.session = self.client.session

    def test_changed_password_doesnt_invalidate_session(self):
        # Changing a user's password shouldn't invalidate the session if session
        # verification isn't activated.
        session_key = self.request.session.session_key
        self.middleware_auth.process_request(self.request)
        self.middleware_session_auth.process_request(self.request)
        self.assertIsNotNone(self.request.user)
        self.assertFalse(self.request.user.is_anonymous())

        # After password change, user should remain logged in.
        self.user.set_password('new_password')
        self.user.save()
        self.middleware_auth.process_request(self.request)
        self.middleware_session_auth.process_request(self.request)
        self.assertIsNotNone(self.request.user)
        self.assertFalse(self.request.user.is_anonymous())
        self.assertEqual(session_key, self.request.session.session_key)

    def test_changed_password_invalidates_session_with_middleware(self):
        session_key = self.request.session.session_key
        with self.modify_settings(MIDDLEWARE_CLASSES={'append': ['django.contrib.auth.middleware.SessionAuthenticationMiddleware']}):
            # After password change, user should be anonymous
            self.user.set_password('new_password')
            self.user.save()
            self.middleware_auth.process_request(self.request)
            self.middleware_session_auth.process_request(self.request)
            self.assertIsNotNone(self.request.user)
            self.assertTrue(self.request.user.is_anonymous())
        # session should be flushed
        self.assertNotEqual(session_key, self.request.session.session_key)


@override_settings(USE_TZ=False, PASSWORD_HASHERS=('django.contrib.auth.hashers.SHA1PasswordHasher',))
class EmailUserCreationFormTest(TestCase):

    def setUp(self):
        get_user_model().objects.create_user('testclient@example.com', 'test123')

    def test_user_already_exists(self):
        data = {
            'email': 'testclient@example.com',
            'password1': 'test123',
            'password2': 'test123',
        }
        form = EmailUserCreationForm(data)
        self.assertFalse(form.is_valid())
        self.assertEqual(form["email"].errors,
                         [force_text(form.error_messages['duplicate_email'])])

    def test_invalid_data(self):
        data = {
            'email': 'testclient',
            'password1': 'test123',
            'password2': 'test123',
        }
        form = EmailUserCreationForm(data)
        self.assertFalse(form.is_valid())
        self.assertEqual(form["email"].errors,
                         [_('Enter a valid email address.')])

    def test_password_verification(self):
        # The verification password is incorrect.
        data = {
            'email': 'testclient@example.com',
            'password1': 'test123',
            'password2': 'test',
        }
        form = EmailUserCreationForm(data)
        self.assertFalse(form.is_valid())
        self.assertEqual(form["password2"].errors,
                         [force_text(form.error_messages['password_mismatch'])])

    def test_both_passwords(self):
        # One (or both) passwords weren't given
        data = {'email': 'testclient@example.com'}
        form = EmailUserCreationForm(data)
        required_error = [force_text(Field.default_error_messages['required'])]
        self.assertFalse(form.is_valid())
        self.assertEqual(form['password1'].errors, required_error)
        self.assertEqual(form['password2'].errors, required_error)

        data['password2'] = 'test123'
        form = EmailUserCreationForm(data)
        self.assertFalse(form.is_valid())
        self.assertEqual(form['password1'].errors, required_error)
        self.assertEqual(form['password2'].errors, [])

    def test_success(self):
        # The success case.
        data = {
            'email': 'jsmith@example.com',
            'password1': 'test123',
            'password2': 'test123',
        }
        form = EmailUserCreationForm(data)
        self.assertTrue(form.is_valid())
        u = form.save()
        self.assertEqual(repr(u), '<%s: jsmith@example.com>' % get_user_model().__name__)
        self.assertIsNotNone(u.pk)

    def test_success_without_commit(self):
        # The success case, but without saving the user instance to the db.
        data = {
            'email': 'jsmith@example.com',
            'password1': 'test123',
            'password2': 'test123',
        }
        form = EmailUserCreationForm(data)
        self.assertTrue(form.is_valid())
        u = form.save(commit=False)
        self.assertEqual(repr(u), '<%s: jsmith@example.com>' % get_user_model().__name__)
        self.assertIsNone(u.pk, None)


@override_settings(USE_TZ=False, PASSWORD_HASHERS=('django.contrib.auth.hashers.SHA1PasswordHasher',))
class EmailUserChangeFormTest(TestCase):

    def setUp(self):
        user = get_user_model().objects.create_user('testclient@example.com')
        user.password = 'sha1$6efc0$f93efe9fd7542f25a7be94871ea45aa95de57161'
        user.save()
        get_user_model().objects.create_user('empty_password@example.com')
        user_unmanageable = get_user_model().objects.create_user('unmanageable_password@example.com')
        user_unmanageable.password = '$'
        user_unmanageable.save()
        user_unknown = get_user_model().objects.create_user('unknown_password@example.com')
        user_unknown.password = 'foo$bar'
        user_unknown.save()

    def test_username_validity(self):
        user = get_user_model().objects.get(email='testclient@example.com')
        data = {'email': 'not valid'}
        form = EmailUserChangeForm(data, instance=user)
        self.assertFalse(form.is_valid())
        self.assertEqual(form['email'].errors,
                         [_('Enter a valid email address.')])

    def test_bug_14242(self):
        # A regression test, introduce by adding an optimization for the
        # EmailUserChangeForm.

        class MyUserForm(EmailUserChangeForm):
            def __init__(self, *args, **kwargs):
                super(MyUserForm, self).__init__(*args, **kwargs)
                self.fields['groups'].help_text = 'These groups give users different permissions'

            class Meta(EmailUserChangeForm.Meta):
                fields = ('groups',)

        # Just check we can create it
        MyUserForm({})

    def test_unsuable_password(self):
        user = get_user_model().objects.get(email='empty_password@example.com')
        user.set_unusable_password()
        user.save()
        form = EmailUserChangeForm(instance=user)
        self.assertIn(_("No password set."), form.as_table())

    def test_bug_17944_empty_password(self):
        user = get_user_model().objects.get(email='empty_password@example.com')
        form = EmailUserChangeForm(instance=user)
        self.assertIn(_("No password set."), form.as_table())

    def test_bug_17944_unmanageable_password(self):
        user = get_user_model().objects.get(email='unmanageable_password@example.com')
        form = EmailUserChangeForm(instance=user)
        self.assertIn(
            _("Invalid password format or unknown hashing algorithm."),
            form.as_table())

    def test_bug_17944_unknown_password_algorithm(self):
        user = get_user_model().objects.get(email='unknown_password@example.com')
        form = EmailUserChangeForm(instance=user)
        self.assertIn(
            _("Invalid password format or unknown hashing algorithm."),
            form.as_table())

    def test_bug_19133(self):
        """The change form does not return the password value."""
        # Use the form to construct the POST data
        user = get_user_model().objects.get(email='testclient@example.com')
        form_for_data = EmailUserChangeForm(instance=user)
        post_data = form_for_data.initial

        # The password field should be readonly, so anything
        # posted here should be ignored; the form will be
        # valid, and give back the 'initial' value for the
        # password field.
        post_data['password'] = 'new password'
        form = EmailUserChangeForm(instance=user, data=post_data)

        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['password'], 'sha1$6efc0$f93efe9fd7542f25a7be94871ea45aa95de57161')

    def test_bug_19349_bound_password_field(self):
        user = get_user_model().objects.get(email='testclient@example.com')
        form = EmailUserChangeForm(data={}, instance=user)
        # When rendering the bound password field,
        # ReadOnlyPasswordHashWidget needs the initial
        # value to render correctly
        self.assertEqual(form.initial['password'], form['password'].value())


class EmailUserAdminTest(TestCase):

    def setUp(self):
        self.user_email = 'test@example.com'
        self.user_password = 'test_password'
        self.user = get_user_model().objects.create_superuser(
            self.user_email,
            self.user_password)

        if settings.AUTH_USER_MODEL == "custom_user.EmailUser":
            self.app_name = "custom_user"
            self.model_name = "emailuser"
            self.model_verbose_name = "user"
            self.model_verbose_name_plural = "Users"
            if django.VERSION[:2] < (1, 7):
                self.app_verbose_name = "Custom_User"
            else:
                self.app_verbose_name = "Custom User"
        if settings.AUTH_USER_MODEL == "test_custom_user_subclass.MyCustomEmailUser":
            self.app_name = "test_custom_user_subclass"
            self.model_name = "mycustomemailuser"
            self.model_verbose_name = "MyCustomEmailUserVerboseName"
            self.model_verbose_name_plural = "MyCustomEmailUserVerboseNamePlural"
            if django.VERSION[:2] < (1, 7):
                self.app_verbose_name = "Test_Custom_User_Subclass"
            else:
                self.app_verbose_name = "Test Custom User Subclass"

    def test_url(self):
        self.assertTrue(self.client.login(
            username=self.user_email,
            password=self.user_password,
        ))
        response = self.client.get(reverse("admin:app_list", args=(self.app_name,)))
        self.assertEqual(response.status_code, 200)

    def test_app_name(self):
        self.assertTrue(self.client.login(
            username=self.user_email,
            password=self.user_password,
        ))

        response = self.client.get(reverse("admin:app_list", args=(self.app_name,)))
        self.assertEqual(response.context['app_list'][0]['name'], self.app_verbose_name)

    def test_model_name(self):
        self.assertTrue(self.client.login(
            username=self.user_email,
            password=self.user_password,
        ))

        response = self.client.get(reverse("admin:%s_%s_changelist" % (self.app_name, self.model_name)))
        self.assertEqual(force_text(response.context['title']), "Select %s to change" % self.model_verbose_name)

    def test_model_name_plural(self):
        self.assertTrue(self.client.login(
            username=self.user_email,
            password=self.user_password,
        ))

        response = self.client.get(reverse("admin:app_list", args=(self.app_name,)))
        self.assertEqual(force_text(response.context['app_list'][0]['models'][0]['name']), self.model_verbose_name_plural)

    def test_user_change_password(self):
        self.assertTrue(self.client.login(
            username=self.user_email,
            password=self.user_password,
        ))

        user_change_url = reverse('admin:%s_%s_change' % (self.app_name, self.model_name), args=(self.user.pk,))

        if django.VERSION[:2] < (1, 8):
            # Since reverse() does not exist yet, create path manually
            password_change_url = user_change_url + 'password/'
        else:
            password_change_url = reverse('admin:auth_user_password_change', args=(self.user.pk,))

        response = self.client.get(user_change_url)
        # Test the link inside password field help_text.
        rel_link = re.search(
            r'you can change the password using <a href="([^"]*)">this form</a>',
            force_text(response.content)
        ).groups()[0]
        self.assertEqual(
            os.path.normpath(user_change_url + rel_link),
            os.path.normpath(password_change_url)
        )

        # Test url is correct.
        self.assertEqual(
            self.client.get(password_change_url).status_code,
            200,
)


"""Django forms for the socialprofile application"""
from django import forms
from django.conf import settings
from models import SocialProfile
from django.core.exceptions import ObjectDoesNotExist
from django.utils.html import strip_tags
from django.utils.translation import ugettext_lazy as _
from widgets import H5EmailInput
from image_cropping import ImageCropWidget
import logging

# pylint: disable=E1120,W0212

LOGGER = logging.getLogger(name='socialprofile.forms')

class SocialProfileForm(forms.ModelForm):
    """Master form for editing the user's profile"""

    # user = forms.IntegerField(widget=forms.HiddenInput, required=True)
    returnTo = forms.CharField(widget=forms.HiddenInput, required=False, initial='/')  # URI to Return to after save
    manually_edited = forms.BooleanField(widget=forms.HiddenInput, required=False, initial=True)
	
    class Meta(object):
        """Configuration for the ModelForm"""
        model = SocialProfile
        fields = ['username', 'first_name', 'last_name', 'email', 'gender', 'url', 'image_url','cropping','cropping_free', 'description',
				  'sort','title',
				  'function_01','function_02','function_03','function_04','function_05','function_06','function_07','function_08','function_09','function_10']
		# Don't let through for security reasons, user should be based on logged in user only

    def clean_description(self):
        """Automatically called by Django, this method 'cleans' the description, e.g. stripping HTML out of desc"""

        LOGGER.debug("socialprofile.forms.SocialProfileForm.clean_description")

        return strip_tags(self.cleaned_data['description'])

    def clean(self):
        """Automatically called by Django, this method 'cleans' the whole form"""

        LOGGER.debug("socialprofile.forms.SocialProfileForm.clean")

        if self.changed_data:
            self.cleaned_data['manually_edited'] = True





from django.contrib.auth import authenticate
from django.utils.translation import ugettext_lazy as _

from socialprofile.models import SocialProfile, Group

from rest_framework import serializers

# Serializers define the API representation.
class SocialProfileSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = SocialProfile
        fields = ('url', 'username', 'email', 'is_staff')
		
class GroupSerializer(serializers.ModelSerializer):
    class Meta:
        model = Group



# -*- coding: utf-8 -*-
"""
Alternative implementation of Django's authentication User model, which allows to authenticate against the OAuth.
This alternative implementation is activated by setting ``AUTH_USER_MODEL = 'socialprofile.SocialProfile'`` in
settings.py, otherwise the default Django or another implementation is used.
"""
from django.contrib.auth import get_user_model
from django.db.models.signals import post_save
from django.db import models
from django.utils import timezone
from django.core.validators import RegexValidator
from django.core.mail import send_mail
from django.core.exceptions import ValidationError
from django.contrib.auth.validators import ASCIIUsernameValidator
from django.contrib.auth.models import BaseUserManager, PermissionsMixin, AbstractBaseUser
from django.contrib.auth.models import Group as BaseGroup
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from image_cropping.fields import ImageRatioField, ImageCropField

import logging
LOGGER = logging.getLogger(name='socialprofile.models')

class IntegerRangeField(models.IntegerField):
	def __init__(self, verbose_name=None, name=None, min_value=None, max_value=None, **kwargs):
		self.min_value, self.max_value = min_value, max_value
		models.IntegerField.__init__(self, verbose_name, name, **kwargs)
	def formfield(self, **kwargs):
		defaults = {'min_value': self.min_value, 'max_value':self.max_value}
		defaults.update(kwargs)
		return super(IntegerRangeField, self).formfield(**defaults)

class SocialProfileManager(BaseUserManager):
    use_in_migrations = True
 
    def get_by_natural_key(self, username):
		try:
			return self.get(username=username)
		except self.model.DoesNotExist:
			return self.get(is_active=True, email=username)
 
    def _create_user(self, email, password,
                     is_staff, is_superuser, **extra_fields):
        """
        Creates and saves a User with the given email and password.
		
		:param str email: user email
        :param str password: user password
        :param bool is_staff: whether user staff or not
        :param bool is_superuser: whether user admin or not
        :return socialprofile.models.SocialProfile user: user
        :raise ValueError: email is not set
		
        """
        now = timezone.localtime(timezone.now())
        if not email:
            raise ValueError('The given email must be set')
        email = self.normalize_email(email)
        user = self.model(email=email,
                          is_staff=is_staff, is_active=True,
                          is_superuser=is_superuser, last_login=now,
                          date_joined=now, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, email, password=None, **extra_fields):
        """
        Create and save a SocialProfile with the given email and password.
        :param str email: user email
        :param str password: user password
        :return socialprofile.models.SocialProfile user: regular user
        """
        is_staff = extra_fields.pop("is_staff", False)
        return self._create_user(email, password, False, False,
                                 **extra_fields)

    def create_superuser(self, email, password, **extra_fields):
        return self._create_user(email, password, True, True,
                                 **extra_fields)

@python_2_unicode_compatible
class AbstractSocialProfile(AbstractBaseUser, PermissionsMixin):
    """
    Alternative implementation of Django's User model allowing to authenticate against the OAuth.
	
    Inherits from both the AbstractBaseUser and PermissionMixin.
    The following attributes are inherited from the superclasses:
        * password
        * last_login
        * is_superuser
    """
 
    username_validator = ASCIIUsernameValidator()    
	
    GENDER_CHOICES = (
        (_('male'), _('Male')),
        (_('female'), _('Female')),
        (_('other'), _('Other')),
        ('', '')
    )
	
    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = ['username']

    username = models.CharField(_('Username'), max_length=30, unique=True, blank=False,
        help_text=_('Required. 30 characters or fewer. Letters, digits and @/./+/-/_ only.'),
        validators=[
            RegexValidator(
                r'^[\w.@+-]+$',
                _('Enter a valid username. This value may contain only '
                  'letters, numbers ' 'and @/./+/-/_ characters.')
            ),
        ],
        error_messages={
            'unique': _("A user with that username already exists."),
        },
    )
    first_name = models.CharField(verbose_name=_('First Name'), max_length=30, blank=True)
    last_name = models.CharField(verbose_name=_('Last Name'), max_length=30, blank=True)
    email = models.EmailField(verbose_name=_('Email Address'), unique=True, max_length=254, blank=False)
    is_staff = models.BooleanField(verbose_name=_('staff status'), default=False,
        help_text=_('Designates whether the user can log into this admin site.'))
    is_active = models.BooleanField(verbose_name=_('active'), default=True,
        help_text=_('Designates whether this user should be treated as active. Unselect this instead of deleting accounts.'))
    date_of_birth = models.DateField(verbose_name=_('Date of Birth'), null=True)
    date_joined = models.DateTimeField(verbose_name=_('Date Joined'), auto_now_add=True)
    date_modified = models.DateTimeField(verbose_name=_('Date Updated'), auto_now=True)
 
    #status = models.ForeignKey(MembershipStatus, on_delete=models.SET_NULL, null=True, default=1)
    country = models.CharField(verbose_name=_("Country"), max_length=255, blank=True)
    gender = models.CharField(verbose_name=_("Gender"), max_length=10, blank=True, choices=GENDER_CHOICES)
    url = models.URLField(verbose_name=_("Homepage"), help_text=_("Where can we find out more about you?"), max_length=500, blank=True)
    image_url = models.URLField(verbose_name=_("Avatar Picture"), max_length=500, blank=True)
    
	# Add Images
    image = ImageCropField(upload_to='user/', blank=True)
    cropping = ImageRatioField('image', '120x100', allow_fullsize=True)
    cropping_free = ImageRatioField('image', '300x300', free_crop=True, size_warning=True)
    description = models.TextField(verbose_name=_("Description"), help_text=_("Tell us about yourself!"), blank=True)
    manually_edited = models.BooleanField(verbose_name=_("Manually Edited"), default=False)

	# Add Contact info
	#max_length=15->'^\+?1?\d{9,15}$'
    phone_regex = RegexValidator(regex=r'^\+\d{8,15}$', message=_("Phone number must be entered in the format: '+999999999'. Up to 15 digits allowed."))
    phone_number = models.CharField(validators=[phone_regex], max_length=16, blank=True)
	
	# Add for Staff Infos
    sort = IntegerRangeField(verbose_name=_("Sort Order"), default=1, min_value=1, max_value=100000000, blank=True)
    visible = models.BooleanField(verbose_name=_("Visible in the Public Pages"), default=False, blank=True)
    title = models.CharField(verbose_name=_("Title"), max_length=500, blank=True)
    role = models.CharField(verbose_name=_("Role"), max_length=500, blank=True)
    function_01 = models.CharField(max_length=200, blank=True)
    function_02 = models.CharField(max_length=200, blank=True)
    function_03 = models.CharField(max_length=200, blank=True)
    function_04 = models.CharField(max_length=200, blank=True)
    function_05 = models.CharField(max_length=200, blank=True)
    function_06 = models.CharField(max_length=200, blank=True)
    function_07 = models.CharField(max_length=200, blank=True)
    function_08 = models.CharField(max_length=200, blank=True)
    function_09 = models.CharField(max_length=200, blank=True)
    function_10 = models.CharField(max_length=200, blank=True)

	# Add Info retrived by Providers
    editByGoogle = models.BooleanField(verbose_name=_("Google Edit User's Info"), default=False, blank=True)
    google_isPlusUser = models.BooleanField(verbose_name=_("Google Plus"), default=False, blank=True)
    google_plusUrl = models.URLField(verbose_name=_("Google Plus Page"), max_length=500, blank=True)
    google_circledByCount = models.IntegerField(verbose_name=_("Google Circle byCount"), default=0, blank=True)
    google_language = models.CharField(verbose_name=_("Google Language"), max_length=2, blank=True)
    google_kind = models.CharField(verbose_name=_("Google Kind"), max_length=2, blank=True)
    google_verified = models.BooleanField(verbose_name=_("Google Verified"), default=False, blank=True)
	
    objects = SocialProfileManager()
    	 
    class Meta(object):
        verbose_name = _("Social Profile")
        verbose_name_plural = _("Social Profiles")
        ordering = ['username']
        proxy = False
        abstract = True

    def save(self, *args, **kwargs):
        if not self.username:
            self.username = self.get_email()
        super(AbstractSocialProfile, self).save(*args, **kwargs)

    def __str__(self):
        return self.get_username()

    @models.permalink
    def get_absolute_url(self):
        return 'sp_profile_other_view_page', [self.username]

    def get_username(self):
        """Return the username"""
        if self.is_staff:
            return self.username
        return self.email or '<anonymous>'

    def get_email(self):
        """Return the email"""
        return self.email

    def get_full_name(self):
        """
        Returns the first_name plus the last_name, with a space in between else email
        """
        full_name = '%s %s' % (self.first_name, self.last_name)
        if full_name:
			return full_name.strip()
        return self.get_email()

    def get_short_name(self):
        """
        Returns the short name for the user.
        """
        return self.first_name

    def email_user(self, subject, message, from_email=None, **kwargs):
        """Send an email to this User."""
        send_mail(subject, message, from_email, [self.email], **kwargs)

class SocialProfile(AbstractSocialProfile):
    """
    Concrete class of AbstractSocialProfile.
    Use this if you don't need to extend SocialProfile.
    """

    class Meta(AbstractSocialProfile.Meta):  # noqa: D101
        swappable = 'AUTH_USER_MODEL'

class Group(BaseGroup):
    class Meta:
        proxy = True

    # def validate_unique(self, exclude=self.id):
        # """
        # Since the email address is used as the primary identifier, we must ensure that it is
        # unique. However, this can not be done on the field declaration since is only applies to
        # active users. Inactive users can not login anyway, so we don't need a unique constraint
        # for them.
        # """
        # super(SocialProfile, self).validate_unique(exclude)
        # if self.email and get_user_model().objects.exclude(id=self.id).filter(is_active=True, email__exact=self.email).exists():
            # msg = _("A customer with the e-mail address ‘{email}’ already exists.")
            # raise ValidationError({'email': msg.format(email=self.email)})

		
		
		
# class UserDetails(models.Model):
    # type = models.OneToOneField('SocialProfile')
    # extra_info = models.CharField(max_length=600)
	
# def create_user_profile(sender, instance, created, **kwargs):
    # """Creates a UserProfile Object Whenever a User Object is Created"""
    # if created:
        # SocialProfile.objects.create(user=instance)


# post_save.connect(create_user_profile, sender=settings.AUTH_USER_MODEL)



"""Django App configuration for socialprofile"""

from django.apps import AppConfig

class SocialProfileConfig(AppConfig):

    """Default configuration for socialprofile"""

    name = 'socialprofile'
    verbose_name = "Social Profile"


"""Django Admin Site configuration for socialprofiles"""

# pylint: disable=R0901,R0904

from django.contrib import admin
from django.contrib.auth.admin import GroupAdmin as BaseGroupAdmin
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from django.contrib.auth.models import Group as StockGroup
from django.utils.translation import ugettext_lazy as _
from socialprofile.models import Group, SocialProfile
from image_cropping.admin import ImageCroppingMixin

@admin.register(SocialProfile)
class CustomUserAdmin(ImageCroppingMixin, BaseUserAdmin):
    """Sets up the custom user admin display"""
    fieldsets = (
        (None, {'fields': ('email', 'username', 'password')}),
        (_('Personal info'), {'fields': ('first_name', 'last_name', 'phone_number')}),
        (_('Permissions'), {'fields': ('is_active', 'is_staff', 'is_superuser',
                                       'groups', 'user_permissions')}),
        (_('Important dates'), {'fields': ('last_login', 'date_joined','date_of_birth')}),
        (_('Contacts Info'), {'fields': ('email','phone_number')}),
        (_('Common Info'), {'fields': ('country', 'gender', 'url', 'image_url')}),
        (_('Staff'), {'fields': ('sort', 'visible', 'title', 'role',
			'function_01','function_02','function_03','function_04','function_05',
			'function_06','function_07','function_08','function_09','function_10',)}),
        (_('Google'), {'fields': ('editByGoogle', 'google_isPlusUser', 'google_plusUrl', 
			'google_circledByCount', 'google_language', 'google_kind', 'google_verified')}),
    )
    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('email', 'email', 'password1', 'password2'),
        }),
    )
    readonly_fields = ['email', 'last_login', 'date_joined', 
		'editByGoogle', 'google_isPlusUser', 'google_plusUrl', 'google_circledByCount', 'google_language', 'google_kind', 'google_verified' ]
    #form = UserChangeForm
    #add_form = UserCreationForm
    list_display = ('email', 'username', 'gender', 'first_name', 'last_name', 'is_active', 'manually_edited','visible','date_joined','last_login')
    search_fields = ('email', 'first_name', 'last_name')
    ordering = ('last_login','date_joined','email',)

from social_django.models import UserSocialAuth, Nonce, Association
from social_django.admin import UserSocialAuthOption, NonceOption, AssociationOption

class ProxyUserSocialAuth(UserSocialAuth):
    """Create UserSocialAuth proxy model for OAuth social_django"""
    class Meta:
        proxy = True   
        verbose_name = "OAuth: " + str(UserSocialAuth._meta.verbose_name)
        verbose_name_plural = "OAuth: " + str(UserSocialAuth._meta.verbose_name_plural)
		
class ProxyNonce(Nonce):
    """Create Nonce proxy model for OAuth social_django"""
    class Meta:
        proxy = True  
        verbose_name = "OAuth: " + str(Nonce._meta.verbose_name)
        verbose_name_plural = "OAuth: " + str(Nonce._meta.verbose_name_plural)		
		
class ProxyAssociation(Association):
    """Create Association proxy model for OAuth social_django"""
    class Meta:
        proxy = True
        verbose_name = "OAuth: " + str(Association._meta.verbose_name)
        verbose_name_plural = "OAuth: " + str(Association._meta.verbose_name_plural)
		
admin.site.unregister(UserSocialAuth)
admin.site.unregister(Nonce)
admin.site.unregister(Association)
admin.site.register(ProxyUserSocialAuth, UserSocialAuthOption)
admin.site.register(ProxyNonce, NonceOption)
admin.site.register(ProxyAssociation, AssociationOption)


from two_factor.models import PhoneDevice
from two_factor.admin import PhoneDeviceAdmin
from django_otp.plugins.otp_static.models import StaticDevice
from django_otp.plugins.otp_static.admin import StaticDeviceAdmin
from django_otp.plugins.otp_totp.models import TOTPDevice
from django_otp.plugins.otp_totp.admin import TOTPDeviceAdmin

class ProxyPhoneDevice(PhoneDevice):
    """Create PhoneDevice proxy model for OAuth two_factor"""
    class Meta:
        proxy = True   
        verbose_name = "Otp: " + str(PhoneDevice._meta.verbose_name)
        verbose_name_plural = "Otp: " + str(PhoneDevice._meta.verbose_name_plural)
		
class ProxyStaticDevice(StaticDevice):
    """Create StaticDevice proxy model for OAuth social_django"""
    class Meta:
        proxy = True  
        verbose_name = "Otp: " + str(StaticDevice._meta.verbose_name)
        verbose_name_plural = "Otp: " + str(StaticDevice._meta.verbose_name_plural)		
		
class ProxyTOTPDevice(TOTPDevice):
    """Create TOTPDevice proxy model for OAuth social_django"""
    class Meta:
        proxy = True
        verbose_name = "Otp: " + str(TOTPDevice._meta.verbose_name)
        verbose_name_plural = "Otp: " + str(TOTPDevice._meta.verbose_name_plural)
		
admin.site.unregister(PhoneDevice)
admin.site.unregister(StaticDevice)
admin.site.unregister(TOTPDevice)
admin.site.register(ProxyPhoneDevice, PhoneDeviceAdmin)
admin.site.register(ProxyStaticDevice, StaticDeviceAdmin)
admin.site.register(ProxyTOTPDevice, TOTPDeviceAdmin)


from user_sessions.admin import SessionAdmin
from user_sessions.models import Session

class ProxySession(Session):
    """Create Session proxy model for user_session"""
    class Meta:
        proxy = True
        verbose_name = "Monitor: " + str(Session._meta.verbose_name)
        verbose_name_plural = "Monitor: " + str(Session._meta.verbose_name_plural)
		
admin.site.unregister(Session)
admin.site.register(ProxySession, SessionAdmin)


from axes.admin import AccessAttemptAdmin, AccessLogAdmin
from axes.models import AccessAttempt, AccessLog

class ProxyAccessAttempt(AccessAttempt):
    """Create AccessAttempt proxy model for user_session"""
    class Meta:
        proxy = True
        verbose_name = "Monitor: " + str(AccessAttempt._meta.verbose_name)
        verbose_name_plural = "Monitor: " + str(AccessAttempt._meta.verbose_name_plural)

class ProxyAccessLog(AccessLog):
    """Create Axes proxy model for user_session"""
    class Meta:
        proxy = True
        verbose_name = "Monitor: " + str(AccessLog._meta.verbose_name)
        verbose_name_plural = "Monitor: " + str(AccessLog._meta.verbose_name_plural)
		
admin.site.unregister(AccessAttempt)
admin.site.unregister(AccessLog)
admin.site.register(ProxyAccessAttempt, AccessAttemptAdmin)
admin.site.register(ProxyAccessLog, AccessLogAdmin)


admin.site.unregister(StockGroup)
@admin.register(Group)
class GroupAdmin(BaseGroupAdmin):
    pass


"""Python Social Auth Pipeline Extensions"""

from social_core.backends.google import GoogleOAuth2 as GoogleOAuth2
from social_core.backends.twitter import TwitterOAuth as TwitterOAuth
from social_core.backends.facebook import FacebookOAuth2 as FacebookOAuth2

def create_profile(user, is_new=False, *args, **kwargs):
    """Create a profile instance for the given user"""
    if is_new:
        profile = user.get_profile()
        if profile is None:
            profile = Profile(user_id=user.id)
        profile.gender = response.get('gender')
        profile.link = response.get('link')
        profile.timezone = response.get('timezone')
        profile.save()

def socialprofile_extra_values(backend, details, response, uid, user, *args, **kwargs):
    """Routes the extra values call to the appropriate back end"""
    if type(backend) is GoogleOAuth2:
        return {'user': google_extra_values(backend, details, response, uid, user, *args, **kwargs) }

    if type(backend) is TwitterOAuth:
        return {'user': twitter_extra_values(backend, details, response, uid, user, *args, **kwargs) }

    if type(backend) is FacebookOAuth2:
        return {'user': facebook_extra_values(backend, details, response, uid, user, *args, **kwargs) }

def google_extra_values(backend, details, response, uid, user, *args, **kwargs):
    """Populates a UserProfile Object when a new User is created via Google Auth"""
    if not user.manually_edited:
        user.last_name = response.get('name').get('familyName', '')
        user.first_name = response.get('name').get('givenName', '')
        user.save()

        user.gender = response.get('gender', '')
        user.image_url = response.get('image').get('url', '')
        user.google_isPlusUser = response.get('isPlusUser', False)
        user.google_plusUrl = response.get('url', '')
        user.google_circledByCount = response.get('circledByCount', 0)
        user.google_language = response.get('language', '')
        user.google_kind = response.get('kind', '')
        user.google_verified = response.get('verified', False)
		
        user.editByGoogle = True
        user.save()
        return user
    return response
	
def facebook_extra_values(backend, details, response, uid, user, *args, **kwargs):
    """Populates a UserProfile Object when a new User is created via Facebook Auth"""
    if not user.manually_edited:
        user.last_name = response.get('last_name', '')
        user.first_name = response.get('first_name', '')
        user.save()

        user.gender = response.get('gender', '')
        user.url = response.get('link', '')
        if response.get('picture', False):
            user.image_url = response.get('picture').get('data').get('url', '')

        user.save()
        return user
    return response

def twitter_extra_values(backend, details, response, uid, user, *args, **kwargs):
    """Populates a UserProfile Object when a new User is created via Twitter Auth"""
    if not user.manually_edited:
        try:
            first_name, last_name = response.get('name', '').split(' ', 1)
        except:
            first_name = response.get('name', '')
            last_name = ''
        user.last_name = last_name
        user.first_name = first_name
        user.save()

        user.url = response.get('url', '')
        user.image_url = response.get('profile_image_url_https', '')
        user.description = response.get('description', '')

        user.save()
        return user
    return response


"""
    Master URL Pattern List for the application.  Most of the patterns here should be top-level
    pass-offs to sub-modules, who will have their own urls.py defining actions within.
"""

# pylint: disable=W0401, W0614, E1120

from django.conf.urls import url, include
from django.views.decorators.cache import never_cache
from django.contrib.auth.decorators import login_required
from django.contrib.auth.views import logout

from . import views

from rest_framework_jwt.views import obtain_jwt_token, refresh_jwt_token, verify_jwt_token

urlpatterns = [ 
	# Profile Self View
	url(r'^$', never_cache(login_required(views.SocialProfileView.as_view())), name="sp_profile_view_page"),
	url(r'^login/$', never_cache(login_required(views.home))),
	url(r'^done/$', views.done, name='done'),
	url(r'^logout/$', views.logout),
	# url(r'^ajax-auth/(?P<backend>[^/]+)/$', views.ajax_auth, name='ajax-auth'),
	
	# Profile Other View
	url(r'^view/(?P<username>\w.+)/$', login_required(views.SocialProfileView.as_view()), name="sp_profile_other_view_page"),

	# Profile Edit
	url(r'^edit/$', never_cache(login_required(views.SocialProfileEditView.as_view())), name="sp_profile_edit_page"),

	# Select Sign Up Method
	url(r'^select/$', never_cache(views.SelectAuthView.as_view()), name="sp_select_page"),

	# Delete
	url(r'^delete/$', login_required(views.DeleteSocialProfileView.as_view()), name="sp_delete_page"),

	# Logout Page
	url(r'^logout/$', logout, kwargs={'next_page': "sp_select_page"}, name="sp_logout_page"),
	
	# Social Auth
	url(r'', include('social_django.urls', namespace='social')),
	
	# User Sessions
	url(r'', include('user_sessions.urls', 'user_sessions')),
	
	# Rest Framework
	url(r'^o/', include('oauth2_provider.urls', namespace='oauth2_provider')),
	url(r'^api-auth/', include('rest_framework.urls', namespace='rest_framework')),
	url(r'^api-token-auth/', obtain_jwt_token),
	url(r'^api-token-refresh/', refresh_jwt_token),
	url(r'^api-token-verify/', verify_jwt_token),
	
	# One-Time-Password
	url(r'', include('two_factor.urls', 'two_factor')),
	
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-08-10 01:57
from __future__ import unicode_literals

from django.db import migrations
import user_sessions.models


class Migration(migrations.Migration):

    dependencies = [
        ('axes', '0003_auto_20160322_0929'),
        ('social_django', '0006_partial'),
        ('otp_totp', '0001_initial'),
        ('user_sessions', '0003_auto_20161205_1516'),
        ('two_factor', '0005_auto_20160224_0450'),
        ('otp_static', '0001_initial'),
        ('socialprofile', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ProxyAccessAttempt',
            fields=[
            ],
            options={
                'verbose_name': 'Monitor: access attempt',
                'proxy': True,
                'verbose_name_plural': 'Monitor: access attempts',
                'indexes': [],
            },
            bases=('axes.accessattempt',),
        ),
        migrations.CreateModel(
            name='ProxyAccessLog',
            fields=[
            ],
            options={
                'verbose_name': 'Monitor: access log',
                'proxy': True,
                'verbose_name_plural': 'Monitor: access logs',
                'indexes': [],
            },
            bases=('axes.accesslog',),
        ),
        migrations.CreateModel(
            name='ProxyAssociation',
            fields=[
            ],
            options={
                'verbose_name': 'OAuth: association',
                'proxy': True,
                'verbose_name_plural': 'OAuth: associations',
                'indexes': [],
            },
            bases=('social_django.association',),
        ),
        migrations.CreateModel(
            name='ProxyNonce',
            fields=[
            ],
            options={
                'verbose_name': 'OAuth: nonce',
                'proxy': True,
                'verbose_name_plural': 'OAuth: nonces',
                'indexes': [],
            },
            bases=('social_django.nonce',),
        ),
        migrations.CreateModel(
            name='ProxyPhoneDevice',
            fields=[
            ],
            options={
                'verbose_name': 'Otp: phone device',
                'proxy': True,
                'verbose_name_plural': 'Otp: phone devices',
                'indexes': [],
            },
            bases=('two_factor.phonedevice',),
        ),
        migrations.CreateModel(
            name='ProxySession',
            fields=[
            ],
            options={
                'verbose_name': 'Monitor: sessione',
                'proxy': True,
                'verbose_name_plural': 'Monitor: sessioni',
                'indexes': [],
            },
            bases=('user_sessions.session',),
            managers=[
                ('objects', user_sessions.models.SessionManager()),
            ],
        ),
        migrations.CreateModel(
            name='ProxyStaticDevice',
            fields=[
            ],
            options={
                'verbose_name': 'Otp: static device',
                'proxy': True,
                'verbose_name_plural': 'Otp: static devices',
                'indexes': [],
            },
            bases=('otp_static.staticdevice',),
        ),
        migrations.CreateModel(
            name='ProxyTOTPDevice',
            fields=[
            ],
            options={
                'verbose_name': 'Otp: TOTP device',
                'proxy': True,
                'verbose_name_plural': 'Otp: TOTP devices',
                'indexes': [],
            },
            bases=('otp_totp.totpdevice',),
        ),
        migrations.CreateModel(
            name='ProxyUserSocialAuth',
            fields=[
            ],
            options={
                'verbose_name': 'OAuth: user social auth',
                'proxy': True,
                'verbose_name_plural': 'OAuth: user social auths',
                'indexes': [],
            },
            bases=('social_django.usersocialauth',),
        ),
    ]
